@@ -69,11 +69,10 @@ void AutomappingManager::autoMapInternal(const QRegion &where,
 {
     mError.clear();
     mWarning.clear();
-    if (!mMapDocument) {
-        mError = tr("No map document found!") + QLatin1Char('\n');
-        emit errorsOccurred();
+    if (!mMapDocument)
         return;
-    }
+
+    const bool automatic = touchedLayer != 0;
 
     if (!mLoaded) {
         const QString mapPath = QFileInfo(mMapDocument->fileName()).path();
@@ -81,7 +80,7 @@ void AutomappingManager::autoMapInternal(const QRegion &where,
         if (loadFile(rulesFileName)) {
             mLoaded = true;
         } else {
-            emit errorsOccurred();
+            emit errorsOccurred(automatic);
             return;
         }
     }
@@ -115,10 +114,10 @@ void AutomappingManager::autoMapInternal(const QRegion &where,
     delete passedRegion;
 
     if (!mWarning.isEmpty())
-        emit warningsOccurred();
+        emit warningsOccurred(automatic);
 
     if (!mError.isEmpty())
-        emit errorsOccurred();
+        emit errorsOccurred(automatic);
 }
 
 bool AutomappingManager::loadFile(const QString &filePath)
