@@ -33,6 +33,11 @@
 #include "addremovetileset.h"
 #include "clipboardmanager.h"
 #include "createobjecttool.h"
+#include "createrectangleobjecttool.h"
+#include "createellipseobjecttool.h"
+#include "createtileobjecttool.h"
+#include "createpolygonobjecttool.h"
+#include "createpolylineobjecttool.h"
 #include "documentmanager.h"
 #include "editpolygontool.h"
 #include "eraser.h"
@@ -285,7 +290,8 @@ MainWindow::MainWindow(QWidget *parent, Qt::WindowFlags flags)
     connect(mUi->actionSave, SIGNAL(triggered()), SLOT(saveFile()));
     connect(mUi->actionSaveAs, SIGNAL(triggered()), SLOT(saveFileAs()));
     connect(mUi->actionSaveAsImage, SIGNAL(triggered()), SLOT(saveAsImage()));
-    connect(mUi->actionExport, SIGNAL(triggered()), SLOT(exportAs()));
+    connect(mUi->actionExport, SIGNAL(triggered()), SLOT(export_()));
+    connect(mUi->actionExportAs, SIGNAL(triggered()), SLOT(exportAs()));
     connect(mUi->actionReload, SIGNAL(triggered()), SLOT(reload()));
     connect(mUi->actionClose, SIGNAL(triggered()), SLOT(closeFile()));
     connect(mUi->actionCloseAll, SIGNAL(triggered()), SLOT(closeAllFiles()));
@@ -367,16 +373,11 @@ MainWindow::MainWindow(QWidget *parent, Qt::WindowFlags flags)
     mStampBrush = new StampBrush(this);
     mTerrainBrush = new TerrainBrush(this);
     mBucketFillTool = new BucketFillTool(this);
-    CreateObjectTool *tileObjectsTool = new CreateObjectTool(
-            CreateObjectTool::CreateTile, this);
-    CreateObjectTool *rectangleObjectsTool = new CreateObjectTool(
-            CreateObjectTool::CreateRectangle, this);
-    CreateObjectTool *ellipseObjectsTool = new CreateObjectTool(
-            CreateObjectTool::CreateEllipse, this);
-    CreateObjectTool *polygonObjectsTool = new CreateObjectTool(
-            CreateObjectTool::CreatePolygon, this);
-    CreateObjectTool *polylineObjectsTool = new CreateObjectTool(
-            CreateObjectTool::CreatePolyline, this);
+    CreateObjectTool *tileObjectsTool = new CreateTileObjectTool(this);
+    CreateObjectTool *rectangleObjectsTool = new CreateRectangleObjectTool(this);
+    CreateObjectTool *ellipseObjectsTool = new CreateEllipseObjectTool(this);
+    CreateObjectTool *polygonObjectsTool = new CreatePolygonObjectTool(this);
+    CreateObjectTool *polylineObjectsTool = new CreatePolylineObjectTool(this);
 
     connect(mTilesetDock, SIGNAL(currentTilesChanged(const TileLayer*)),
             this, SLOT(setStampBrush(const TileLayer*)));
@@ -493,10 +494,10 @@ MainWindow::MainWindow(QWidget *parent, Qt::WindowFlags flags)
     readSettings();
     setupQuickStamps();
 
-    connect(mAutomappingManager, SIGNAL(warningsOccurred()),
-            this, SLOT(autoMappingWarning()));
-    connect(mAutomappingManager, SIGNAL(errorsOccurred()),
-            this, SLOT(autoMappingError()));
+    connect(mAutomappingManager, SIGNAL(warningsOccurred(bool)),
+            this, SLOT(autoMappingWarning(bool)));
+    connect(mAutomappingManager, SIGNAL(errorsOccurred(bool)),
+            this, SLOT(autoMappingError(bool)));
 }
 
 MainWindow::~MainWindow()
@@ -510,6 +511,8 @@ MainWindow::~MainWindow()
     mTileCollisionEditor->setTile(0);
     mTileCollisionEditor->writeSettings();
 
+    delete mQuickStampManager;
+
     TilesetManager::deleteInstance();
     DocumentManager::deleteInstance();
     Preferences::deleteInstance();
@@ -852,6 +855,42 @@ void MainWindow::saveAsImage()
     dialog.exec();
 }
 
+void MainWindow::export_()
+{
+    if (!mMapDocument)
+        return;
+
+    QString exportFileName = mMapDocument->lastExportFileName();
+    QString exportPluginFileName = mMapDocument->exportPluginFileName();
+    TmxMapWriter mapWriter;
+
+    if (!exportFileName.isEmpty()) {
+        MapWriterInterface *writer = 0;
+
+        if (exportPluginFileName.isEmpty()) {
+            writer = &mapWriter;
+        } else {
+            PluginManager *pm = PluginManager::instance();
+            if (const Plugin *plugin = pm->pluginByFileName(exportPluginFileName))
+                writer = qobject_cast<MapWriterInterface*>(plugin->instance);
+        }
+
+        if (writer) {
+            if (writer->write(mMapDocument->map(), exportFileName)) {
+                statusBar()->showMessage(tr("Exported to %1").arg(exportFileName),
+                                         3000);
+                return;
+            }
+
+            QMessageBox::critical(this, tr("Error Exporting Map"),
+                                  writer->errorString());
+        }
+    }
+
+    // fall back when no succesful export happened
+    exportAs();
+}
+
 void MainWindow::exportAs()
 {
     if (!mMapDocument)
@@ -869,22 +908,26 @@ void MainWindow::exportAs()
         }
     }
 
+    Preferences *pref = Preferences::instance();
+
     QString selectedFilter =
             mSettings.value(QLatin1String("lastUsedExportFilter")).toString();
+    QString suggestedFilename = mMapDocument->lastExportFileName();
 
-    QFileInfo baseNameInfo = QFileInfo(mMapDocument->fileName());
-    QString baseName = baseNameInfo.baseName();
+    if (suggestedFilename.isEmpty()) {
+        QFileInfo baseNameInfo = QFileInfo(mMapDocument->fileName());
+        QString baseName = baseNameInfo.baseName();
 
-    QRegExp extensionFinder(QLatin1String("\\(\\*\\.([^\\)\\s]*)"));
-    extensionFinder.indexIn(selectedFilter);
-    const QString extension = extensionFinder.cap(1);
+        QRegExp extensionFinder(QLatin1String("\\(\\*\\.([^\\)\\s]*)"));
+        extensionFinder.indexIn(selectedFilter);
+        const QString extension = extensionFinder.cap(1);
 
-    Preferences *pref = Preferences::instance();
-    QString lastExportedFilePath = pref->lastPath(Preferences::ExportedFile);
+        QString lastExportedFilePath = pref->lastPath(Preferences::ExportedFile);
 
-    QString suggestedFilename = lastExportedFilePath
-                                + QLatin1String("/") + baseName
-                                + QLatin1Char('.') + extension;
+        suggestedFilename = lastExportedFilePath
+                + QLatin1String("/") + baseName
+                + QLatin1Char('.') + extension;
+    }
 
     QString fileName = QFileDialog::getSaveFileName(this, tr("Export As..."),
                                                     suggestedFilename,
@@ -938,8 +981,15 @@ void MainWindow::exportAs()
     mSettings.setValue(QLatin1String("lastUsedExportFilter"), selectedFilter);
 
     if (!chosenWriter->write(mMapDocument->map(), fileName)) {
-        QMessageBox::critical(this, tr("Error Saving Map"),
+        QMessageBox::critical(this, tr("Error Exporting Map"),
                               chosenWriter->errorString());
+    } else {
+        // Remember export parameters, so subsequent exports can be done faster
+        mMapDocument->setLastExportFileName(fileName);
+        QString exportPluginFileName;
+        if (const Plugin *plugin = pm->plugin(chosenWriter))
+            exportPluginFileName = plugin->fileName;
+        mMapDocument->setExportPluginFileName(exportPluginFileName);
     }
 }
 
@@ -1224,20 +1274,28 @@ void MainWindow::editMapProperties()
     mMapDocument->emitEditCurrentObject();
 }
 
-void MainWindow::autoMappingError()
+void MainWindow::autoMappingError(bool automatic)
 {
     const QString title = tr("Automatic Mapping Error");
     QString error = mAutomappingManager->errorString();
-    if (!error.isEmpty())
-        QMessageBox::critical(this, title, error);
+    if (!error.isEmpty()) {
+        if (automatic)
+            statusBar()->showMessage(error, 3000);
+        else
+            QMessageBox::critical(this, title, error);
+    }
 }
 
-void MainWindow::autoMappingWarning()
+void MainWindow::autoMappingWarning(bool automatic)
 {
     const QString title = tr("Automatic Mapping Warning");
-    QString warnings = mAutomappingManager->warningString();
-    if (!warnings.isEmpty())
-        QMessageBox::warning(this, title, warnings);
+    QString warning = mAutomappingManager->warningString();
+    if (!warning.isEmpty()) {
+        if (automatic)
+            statusBar()->showMessage(warning, 3000);
+        else
+            QMessageBox::warning(this, title, warning);
+    }
 }
 
 void MainWindow::onAnimationEditorClosed()
@@ -1344,6 +1402,8 @@ void MainWindow::updateActions()
     mUi->actionSaveAs->setEnabled(map);
     mUi->actionSaveAsImage->setEnabled(map);
     mUi->actionExport->setEnabled(map);
+    mUi->actionExportAs->setEnabled(map);
+    mUi->actionReload->setEnabled(map);
     mUi->actionClose->setEnabled(map);
     mUi->actionCloseAll->setEnabled(map);
     mUi->actionCut->setEnabled(canCopy);
