@@ -27,6 +27,7 @@
 #include "changeproperties.h"
 #include "changeselectedarea.h"
 #include "flipmapobjects.h"
+#include "hexagonalrenderer.h"
 #include "imagelayer.h"
 #include "isometricrenderer.h"
 #include "layermodel.h"
@@ -66,21 +67,12 @@ MapDocument::MapDocument(Map *map, const QString &fileName):
     mMap(map),
     mLayerModel(new LayerModel(this)),
     mCurrentObject(map),
+    mRenderer(0),
     mMapObjectModel(new MapObjectModel(this)),
     mTerrainModel(new TerrainModel(this, this)),
     mUndoStack(new QUndoStack(this))
 {
-    switch (map->orientation()) {
-    case Map::Isometric:
-        mRenderer = new IsometricRenderer(map);
-        break;
-    case Map::Staggered:
-        mRenderer = new StaggeredRenderer(map);
-        break;
-    default:
-        mRenderer = new OrthogonalRenderer(map);
-        break;
-    }
+    createRenderer();
 
     mCurrentLayerIndex = (map->layerCount() == 0) ? -1 : 0;
     mLayerModel->setMapDocument(this);
@@ -617,6 +609,12 @@ void MapDocument::setSelectedObjects(const QList<MapObject *> &selectedObjects)
         setCurrentObject(selectedObjects.first());
 }
 
+void MapDocument::setSelectedTiles(const QList<Tile*> &selectedTiles)
+{
+    mSelectedTiles = selectedTiles;
+    emit selectedTilesChanged();
+}
+
 void MapDocument::setCurrentObject(Object *object)
 {
     if (object == mCurrentObject)
@@ -912,3 +910,24 @@ void MapDocument::removeProperty(Object *object, const QString &name)
     object->removeProperty(name);
     emit propertyRemoved(object, name);
 }
+
+void MapDocument::createRenderer()
+{
+    if (mRenderer)
+        delete mRenderer;
+
+    switch (mMap->orientation()) {
+    case Map::Isometric:
+        mRenderer = new IsometricRenderer(mMap);
+        break;
+    case Map::Staggered:
+        mRenderer = new StaggeredRenderer(mMap);
+        break;
+    case Map::Hexagonal:
+        mRenderer = new HexagonalRenderer(mMap);
+        break;
+    default:
+        mRenderer = new OrthogonalRenderer(mMap);
+        break;
+    }
+}
