@@ -110,13 +110,17 @@ public:
 
     QString fileName() const { return mFileName; }
 
-    QString readerPluginFileName() const { return mReaderPluginFileName; }
-    void setReaderPluginFileName(const QString &readerPluginFileName)
-    { mReaderPluginFileName = readerPluginFileName; }
+    QString lastExportFileName() const;
+    void setLastExportFileName(const QString &fileName);
 
-    QString writerPluginFileName() const { return mWriterPluginFileName; }
-    void setWriterPluginFileName(const QString &writerPluginFileName)
-    { mWriterPluginFileName = writerPluginFileName; }
+    QString readerPluginFileName() const;
+    void setReaderPluginFileName(const QString &fileName);
+
+    QString writerPluginFileName() const;
+    void setWriterPluginFileName(const QString &fileName);
+
+    QString exportPluginFileName() const;
+    void setExportPluginFileName(const QString &fileName);
 
     QString displayName() const;
 
@@ -204,6 +208,12 @@ public:
      */
     MapRenderer *renderer() const { return mRenderer; }
 
+    /**
+     * Creates the map renderer. Should be called after changing the map
+     * orientation.
+     */
+    void createRenderer();
+
     /**
      * Returns the undo stack of this map document. Should be used to push any
      * commands on that modify the map.
@@ -238,8 +248,7 @@ public:
     const QList<Tile*> &selectedTiles() const
     { return mSelectedTiles; }
 
-    void setSelectedTiles(const QList<Tile*> &selectedTiles)
-    { mSelectedTiles = selectedTiles; }
+    void setSelectedTiles(const QList<Tile*> &selectedTiles);
 
     Object *currentObject() const { return mCurrentObject; }
     void setCurrentObject(Object *object);
@@ -262,6 +271,7 @@ public:
     void emitMapChanged();
     void emitRegionChanged(const QRegion &region);
     void emitRegionEdited(const QRegion &region, Layer *layer);
+    void emitTileLayerDrawMarginsChanged(TileLayer *layer);
     void emitTilesetChanged(Tileset *tileset);
     void emitTileTerrainChanged(const QList<Tile*> &tiles);
     void emitTileObjectGroupChanged(Tile *tile);
@@ -290,6 +300,11 @@ signals:
      */
     void selectedObjectsChanged();
 
+    /**
+     * Emitted when the list of selected tiles from the dock changes.
+     */
+    void selectedTilesChanged();
+
     void currentObjectChanged(Object *object);
 
     /**
@@ -329,6 +344,8 @@ signals:
      */
     void regionEdited(const QRegion &region, Layer *layer);
 
+    void tileLayerDrawMarginsChanged(TileLayer *layer);
+
     /**
      * Emitted when the terrain information for the given list of tiles was
      * changed. All the tiles are guaranteed to be from the same tileset.
@@ -396,6 +413,7 @@ private:
     void deselectObjects(const QList<MapObject*> &objects);
 
     QString mFileName;
+    QString mLastExportFileName;
 
     /*
      * The filename of a plugin is unique. So it can be used to determine
@@ -406,6 +424,7 @@ private:
      */
     QString mReaderPluginFileName;
     QString mWriterPluginFileName;
+    QString mExportPluginFileName;
     Map *mMap;
     LayerModel *mLayerModel;
     QRegion mSelectedArea;
@@ -420,6 +439,46 @@ private:
     QDateTime mLastSaved;
 };
 
+inline QString MapDocument::lastExportFileName() const
+{
+    return mLastExportFileName;
+}
+
+inline void MapDocument::setLastExportFileName(const QString &fileName)
+{
+    mLastExportFileName = fileName;
+}
+
+inline QString MapDocument::readerPluginFileName() const
+{
+    return mReaderPluginFileName;
+}
+
+inline void MapDocument::setReaderPluginFileName(const QString &fileName)
+{
+    mReaderPluginFileName = fileName;
+}
+
+inline QString MapDocument::writerPluginFileName() const
+{
+    return mWriterPluginFileName;
+}
+
+inline void MapDocument::setWriterPluginFileName(const QString &fileName)
+{
+    mWriterPluginFileName = fileName;
+}
+
+inline QString MapDocument::exportPluginFileName() const
+{
+    return mExportPluginFileName;
+}
+
+inline void MapDocument::setExportPluginFileName(const QString &fileName)
+{
+    mExportPluginFileName = fileName;
+}
+
 /**
  * Emits the map changed signal. This signal should be emitted after changing
  * the map size or its tile size.
@@ -448,6 +507,11 @@ inline void MapDocument::emitRegionEdited(const QRegion &region, Layer *layer)
     emit regionEdited(region, layer);
 }
 
+inline void MapDocument::emitTileLayerDrawMarginsChanged(TileLayer *layer)
+{
+    emit tileLayerDrawMarginsChanged(layer);
+}
+
 /**
  * Emits the signal notifying tileset models about changes to tile terrain
  * information. All the \a tiles need to be from the same tileset.
