@@ -24,7 +24,7 @@
 #include "changeimagelayerposition.h"
 #include "changeimagelayerproperties.h"
 #include "changemapobject.h"
-#include "changemapproperties.h"
+#include "changemapproperty.h"
 #include "changeobjectgroupproperties.h"
 #include "changeproperties.h"
 #include "flipmapobjects.h"
@@ -69,6 +69,17 @@ PropertyBrowser::PropertyBrowser(QWidget *parent)
     setRootIsDecorated(false);
     setPropertiesWithoutValueMarked(true);
 
+    mStaggerAxisNames.append(tr("X"));
+    mStaggerAxisNames.append(tr("Y"));
+
+    mStaggerIndexNames.append(tr("Odd"));
+    mStaggerIndexNames.append(tr("Even"));
+
+    mOrientationNames.append(QCoreApplication::translate("Tiled::Internal::NewMapDialog", "Orthogonal"));
+    mOrientationNames.append(QCoreApplication::translate("Tiled::Internal::NewMapDialog", "Isometric"));
+    mOrientationNames.append(QCoreApplication::translate("Tiled::Internal::NewMapDialog", "Isometric (Staggered)"));
+    mOrientationNames.append(QCoreApplication::translate("Tiled::Internal::NewMapDialog", "Hexagonal (Staggered)"));
+
     mLayerFormatNames.append(QCoreApplication::translate("PreferencesDialog", "XML"));
     mLayerFormatNames.append(QCoreApplication::translate("PreferencesDialog", "Base64 (uncompressed)"));
     mLayerFormatNames.append(QCoreApplication::translate("PreferencesDialog", "Base64 (gzip compressed)"));
@@ -178,6 +189,10 @@ void PropertyBrowser::setMapDocument(MapDocument *mapDocument)
                 SLOT(propertyChanged(Object*,QString)));
         connect(mapDocument, SIGNAL(propertiesChanged(Object*)),
                 SLOT(propertiesChanged(Object*)));
+        connect(mapDocument, SIGNAL(selectedObjectsChanged()),
+                SLOT(selectedObjectsChanged()));
+        connect(mapDocument, SIGNAL(selectedTilesChanged()),
+                SLOT(selectedTilesChanged()));
     }
 }
 
@@ -242,27 +257,54 @@ void PropertyBrowser::terrainChanged(Tileset *tileset, int index)
 
 void PropertyBrowser::propertyAdded(Object *object, const QString &name)
 {
-    if (mObject != object)
+    if (!mMapDocument->currentObjects().contains(object))
         return;
+    if (mNameToProperty.keys().contains(name)) {
+        if (mObject == object) {
+            mUpdating = true;
+            mNameToProperty[name]->setValue(mObject->property(name));
+            mUpdating = false;
+        }
+    } else {
+        // Determine the property preceding the new property, if any
+        const int index = mObject->properties().keys().indexOf(name);
+        const QList<QtProperty *> properties = mCustomPropertiesGroup->subProperties();
+        QtProperty *precedingProperty = (index > 0) ? properties.at(index - 1) : 0;
 
-    // Determine the property preceding the new property, if any
-    const int index = mObject->properties().keys().indexOf(name);
-    const QList<QtProperty *> properties = mCustomPropertiesGroup->subProperties();
-    QtProperty *precedingProperty = (index > 0) ? properties.at(index - 1) : 0;
-
-    mUpdating = true;
-    QtVariantProperty *property = mVariantManager->addProperty(QVariant::String, name);
-    property->setValue(object->property(name));
-    mCustomPropertiesGroup->insertSubProperty(property, precedingProperty);
-    mPropertyToId.insert(property, CustomProperty);
-    mNameToProperty.insert(name, property);
-    mUpdating = false;
+        mUpdating = true;
+        QtVariantProperty *property = mVariantManager->addProperty(QVariant::String, name);
+        property->setValue(mObject->property(name));
+        mCustomPropertiesGroup->insertSubProperty(property, precedingProperty);
+        mPropertyToId.insert(property, CustomProperty);
+        mNameToProperty.insert(name, property);
+        mUpdating = false;
+    }
+    updatePropertyColor(name);
 }
 
 void PropertyBrowser::propertyRemoved(Object *object, const QString &name)
 {
-    if (mObject == object)
-        delete mNameToProperty.take(name);
+    if (!mMapDocument->currentObjects().contains(object))
+        return;
+    if (mObject == object) {
+        bool deleteProperty = true;
+        foreach (Object *obj, mMapDocument->currentObjects()) {
+            if (mObject == obj)
+                continue;
+            if (obj->properties().contains(name)) {
+                // An other selected object still has this property, so just clear the value.
+                mUpdating = true;
+                mNameToProperty[name]->setValue(tr(""));
+                mUpdating = false;
+                deleteProperty = false;
+                break;
+            }
+        }
+        // No other selected objects have this property so delete it.
+        if (deleteProperty)
+            delete mNameToProperty.take(name);
+    }
+    updatePropertyColor(name);
 }
 
 void PropertyBrowser::propertyChanged(Object *object, const QString &name)
@@ -272,14 +314,26 @@ void PropertyBrowser::propertyChanged(Object *object, const QString &name)
         mNameToProperty[name]->setValue(object->property(name));
         mUpdating = false;
     }
+    if (mMapDocument->currentObjects().contains(object))
+        updatePropertyColor(name);
 }
 
 void PropertyBrowser::propertiesChanged(Object *object)
 {
-    if (mObject == object)
+    if (mMapDocument->currentObjects().contains(object))
         updateCustomProperties();
 }
 
+void PropertyBrowser::selectedObjectsChanged()
+{
+    updateCustomProperties();
+}
+
+void PropertyBrowser::selectedTilesChanged()
+{
+    updateCustomProperties();
+}
+
 void PropertyBrowser::valueChanged(QtProperty *property, const QVariant &val)
 {
     if (mUpdating)
@@ -314,6 +368,35 @@ void PropertyBrowser::addMapProperties()
 {
     QtProperty *groupProperty = mGroupManager->addProperty(tr("Map"));
 
+    createProperty(SizeProperty, QVariant::Size, tr("Size"), groupProperty)->setEnabled(false);
+    createProperty(TileSizeProperty, QVariant::Size, tr("Tile Size"), groupProperty);
+
+    QtVariantProperty *orientationProperty =
+            createProperty(OrientationProperty,
+                           QtVariantPropertyManager::enumTypeId(),
+                           tr("Orientation"),
+                           groupProperty);
+
+    orientationProperty->setAttribute(QLatin1String("enumNames"), mOrientationNames);
+
+    createProperty(HexSideLengthProperty, QVariant::Int, tr("Tile Side Length (Hex)"), groupProperty);
+
+    QtVariantProperty *staggerAxisProperty =
+            createProperty(StaggerAxisProperty,
+                           QtVariantPropertyManager::enumTypeId(),
+                           tr("Stagger Axis"),
+                           groupProperty);
+
+    staggerAxisProperty->setAttribute(QLatin1String("enumNames"), mStaggerAxisNames);
+
+    QtVariantProperty *staggerIndexProperty =
+            createProperty(StaggerIndexProperty,
+                           QtVariantPropertyManager::enumTypeId(),
+                           tr("Stagger Index"),
+                           groupProperty);
+
+    staggerIndexProperty->setAttribute(QLatin1String("enumNames"), mStaggerIndexNames);
+
     QtVariantProperty *layerFormatProperty =
             createProperty(LayerFormatProperty,
                            QtVariantPropertyManager::enumTypeId(),
@@ -345,6 +428,8 @@ static QStringList objectTypeNames()
 void PropertyBrowser::addMapObjectProperties()
 {
     QtProperty *groupProperty = mGroupManager->addProperty(tr("Object"));
+
+    createProperty(IdProperty, QVariant::Int, tr("ID"), groupProperty)->setEnabled(false);
     createProperty(NameProperty, QVariant::String, tr("Name"), groupProperty);
 
     QtVariantProperty *typeProperty =
@@ -445,33 +530,55 @@ void PropertyBrowser::addTerrainProperties()
 
 void PropertyBrowser::applyMapValue(PropertyId id, const QVariant &val)
 {
-    Map *map = static_cast<Map*>(mObject);
     QUndoCommand *command = 0;
 
     switch (id) {
+    case TileSizeProperty: {
+        const Map *map = static_cast<Map*>(mObject);
+        const QSize tileSize = val.toSize();
+        if (tileSize.width() != map->tileWidth()) {
+            command = new ChangeMapProperty(mMapDocument,
+                                            ChangeMapProperty::TileWidth,
+                                            tileSize.width());
+        } else if (tileSize.height() != map->tileHeight()) {
+            command = new ChangeMapProperty(mMapDocument,
+                                            ChangeMapProperty::TileHeight,
+                                            tileSize.height());
+        }
+        break;
+    }
+    case OrientationProperty: {
+        Map::Orientation orientation = static_cast<Map::Orientation>(val.toInt() + 1);
+        command = new ChangeMapProperty(mMapDocument, orientation);
+        break;
+    }
+    case HexSideLengthProperty: {
+        command = new ChangeMapProperty(mMapDocument, ChangeMapProperty::HexSideLength,
+                                        val.toInt());
+        break;
+    }
+    case StaggerAxisProperty: {
+        Map::StaggerAxis staggerAxis = static_cast<Map::StaggerAxis>(val.toInt());
+        command = new ChangeMapProperty(mMapDocument, staggerAxis);
+        break;
+    }
+    case StaggerIndexProperty: {
+        Map::StaggerIndex staggerIndex = static_cast<Map::StaggerIndex>(val.toInt());
+        command = new ChangeMapProperty(mMapDocument, staggerIndex);
+        break;
+    }
     case LayerFormatProperty: {
         Map::LayerDataFormat format = static_cast<Map::LayerDataFormat>(val.toInt());
-        command = new ChangeMapProperties(mMapDocument,
-                                          map->backgroundColor(),
-                                          format,
-                                          map->renderOrder());
+        command = new ChangeMapProperty(mMapDocument, format);
         break;
     }
-
     case RenderOrderProperty: {
-            Map::RenderOrder renderOrder = static_cast<Map::RenderOrder>(val.toInt());
-            command = new ChangeMapProperties(mMapDocument,
-                                              map->backgroundColor(),
-                                              map->layerDataFormat(),
-                                              renderOrder);
-            break;
-        }
-
+        Map::RenderOrder renderOrder = static_cast<Map::RenderOrder>(val.toInt());
+        command = new ChangeMapProperty(mMapDocument, renderOrder);
+        break;
+    }
     case ColorProperty:
-        command = new ChangeMapProperties(mMapDocument,
-                                          val.value<QColor>(),
-                                          map->layerDataFormat(),
-                                          map->renderOrder());
+        command = new ChangeMapProperty(mMapDocument, val.value<QColor>());
         break;
     default:
         break;
@@ -726,6 +833,12 @@ void PropertyBrowser::updateProperties()
     switch (mObject->typeId()) {
     case Object::MapType: {
         const Map *map = static_cast<const Map*>(mObject);
+        mIdToProperty[SizeProperty]->setValue(map->size());
+        mIdToProperty[TileSizeProperty]->setValue(map->tileSize());
+        mIdToProperty[OrientationProperty]->setValue(map->orientation() - 1);
+        mIdToProperty[HexSideLengthProperty]->setValue(map->hexSideLength());
+        mIdToProperty[StaggerAxisProperty]->setValue(map->staggerAxis());
+        mIdToProperty[StaggerIndexProperty]->setValue(map->staggerIndex());
         mIdToProperty[LayerFormatProperty]->setValue(map->layerDataFormat());
         mIdToProperty[RenderOrderProperty]->setValue(map->renderOrder());
         QColor backgroundColor = map->backgroundColor();
@@ -736,6 +849,7 @@ void PropertyBrowser::updateProperties()
     }
     case Object::MapObjectType: {
         const MapObject *mapObject = static_cast<const MapObject*>(mObject);
+        mIdToProperty[IdProperty]->setValue(mapObject->id());
         mIdToProperty[NameProperty]->setValue(mapObject->name());
         mIdToProperty[TypeProperty]->setValue(mapObject->type());
         mIdToProperty[VisibleProperty]->setValue(mapObject->isVisible());
@@ -804,12 +918,32 @@ void PropertyBrowser::updateProperties()
 
 void PropertyBrowser::updateCustomProperties()
 {
+    if (!mObject)
+        return;
+
     mUpdating = true;
 
     qDeleteAll(mNameToProperty);
     mNameToProperty.clear();
 
-    QMapIterator<QString,QString> it(mObject->properties());
+    mCombinedProperties = mObject->properties();
+    // Add properties from selected objects which mObject does not contain to mCombinedProperties.
+    foreach (Object *obj, mMapDocument->currentObjects()) {
+        if (obj == mObject)
+            continue;
+
+        QMapIterator<QString,QString> it(obj->properties());
+
+        while (it.hasNext()) {
+            it.next();
+            if (!mCombinedProperties.contains(it.key())) {
+                mCombinedProperties.insert(it.key(), tr(""));
+            }
+        }
+    }
+
+    QMapIterator<QString,QString> it(mCombinedProperties);
+
     while (it.hasNext()) {
         it.next();
         QtVariantProperty *property = createProperty(CustomProperty,
@@ -817,10 +951,44 @@ void PropertyBrowser::updateCustomProperties()
                                                      it.key(),
                                                      mCustomPropertiesGroup);
         property->setValue(it.value());
+        updatePropertyColor(it.key());
     }
 
     mUpdating = false;
 }
 
+// If there are other objects selected check if their properties are equal. If not give them a gray color.
+void PropertyBrowser::updatePropertyColor(const QString &name)
+{
+    QtVariantProperty *property = mNameToProperty.value(name);
+    if (!property)
+        return;
+
+    QString propertyName = property->propertyName();
+    QString propertyValue = property->valueText();
+
+    // If one of the objects doesn't have this property then gray out the name and value.
+    foreach (Object *obj, mMapDocument->currentObjects()) {
+        if (!obj->hasProperty(propertyName)) {
+            property->setNameColor(Qt::gray);
+            property->setValueColor(Qt::gray);
+            return;
+        }
+    }
+
+    // If one of the objects doesn't have the same property value then gray out the value.
+    foreach (Object *obj, mMapDocument->currentObjects()) {
+        if (obj == mObject)
+            continue;
+        if (obj->property(propertyName) != propertyValue) {
+            property->setValueColor(Qt::gray);
+            return;
+        }
+    }
+
+    property->setNameColor(Qt::black);
+    property->setValueColor(Qt::black);
+}
+
 } // namespace Internal
 } // namespace Tiled
